﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Diagnostics;

namespace Kaushik.Spot.CodeGenerator
{
    public static class ProxyBody
    {

        private const short _bufferSize = 5120;
        private const int _socketClientTimeout = 50000000;
        private const short _chunkSize = 500;

        private static string _templateHeaderComment = "/**********************************************************************************\n This class is generated using Kaushik.Spot.CodeGenerator, \n any change to this file may lead to malfunction.\n **********************************************************************************/\n";

        /* Above string will take following inputs
         * {0} Namespace of Proxy Class
         * {1} Name of Proxy Class
         * {2} Name of Proxy Class
         * {3} Server IP of Service Host
         * {4} Communication Port
         * {5} Max Buffer Size
         * {6} Socket Timeout
         * {7} Chunk Size
         * {8} Full Service Name i.e. <Namespace>.<Class>
         */
        private static string _templateClassBegin = "using System;\nusing System.Net.Sockets;\nusing Kaushik.Spot.Library;\n\nnamespace {0}\n{{\n\tpublic class {1}\n\t{{\n\t\tprivate string _serverIp;\n\t\tprivate short _port;\n\t\tprivate short _bufferSize;\n\t\tprivate int _socketClientTimeout;\n\t\tprivate short _chunkSize;\n\t\tprivate string _serviceName;\n\t\tprivate ICryptographicServiceProvider _cipher;\n\t\tprivate ISpotSocketClient _socketClient;\n\n\n\t\tpublic {2}(ICryptographicServiceProvider cipher = null, SpotSocketProvider provider = SpotSocketProvider.Tcp,\n\t\t\tstring serverIp = \"{3}\", short port = {4})\n\t\t{{\n\t\t\t_serverIp = serverIp;\n\t\t\t_port = port;\n\t\t\t_bufferSize = {5};\n\t\t\t_socketClientTimeout = {6};\n\t\t\t_chunkSize = {7};\n\t\t\t_serviceName = \"{8}\";\n\t\t\t_cipher = cipher;\n\t\t\t_socketClient = SpotSocketFactory.GetClientProvider(_serverIp, provider, _port, _bufferSize, _socketClientTimeout, _chunkSize);\n\t\t}}\n\n\n\t\tprivate string encrypt(string plainText)\n\t\t{{\n\t\t\tif (_cipher == null)\n\t\t\t{{\n\t\t\t\treturn plainText;\n\t\t\t}}\n\t\t\telse\n\t\t\t{{\n\t\t\t\treturn _cipher.Encrypt(plainText);\n\t\t\t}}\n\t\t}}\n\n\n\t\tprivate string decrypt(string cipherText)\n\t\t{{\n\t\t\tif (_cipher == null)\n\t\t\t{{\n\t\t\t\treturn cipherText;\n\t\t\t}}\n\t\t\telse\n\t\t\t{{\n\t\t\t\treturn _cipher.Decrypt(cipherText);\n\t\t\t}}\n\t\t}}";

        /* Above string will take following inputs
         * {0} Function Return Type
         * {1} Function Name
         * {2} Ordered Comma Separated Parameters Name with type
         * {3} Function Return Type
         * {4} Defalt value for return type
         * {5} Function Name
         * {6} Ordered Comma Separated Parameters Name
         * {7} Convert Result into return type
         */
        private static string _templateMethodBody = "\n\t\tpublic {0} {1}({2})\n\t\t{{\n\t\t\tstring request = null;\n\t\t\tServiceCommand command = new ServiceCommand();\n\t\t\tServiceResult result = new ServiceResult();\n\t\t\tstring response;\n\t\t\t{3} finalResult = {4};\n\t\t\tusing (Socket socket = _socketClient.CreateClient())\n\t\t\t{{\n\t\t\t\tcommand.ServiceName = _serviceName;\n\t\t\t\tcommand.Method = \"{5}\";\n\t\t\t\tcommand.Parameters = new object[] {{ {6} }};\n\t\t\t\trequest = encrypt(command.Serialize());\n\t\t\t\t_socketClient.Send(request);\n\t\t\t\t_socketClient.Receive(out response);\n\t\t\t\tresult.Deserialize(decrypt(response));\n\t\t\t\tfinalResult = {7}(result.Result);\n\t\t\t\tif (!String.IsNullOrEmpty(result.Error))\n\t\t\t\t{{\n\t\t\t\t\tthrow new Exception(result.Error);\n\t\t\t\t}}\n\t\t\t}}\n\t\t\treturn finalResult;\n\t\t}}\n";

        private static string _templateClassEnd = "\n\t}}\n}}";

        public static string GenerateProxy(Type service, string namespaceName, string proxyClassName, string serverIpOrHostName, int port)
        {
            StringBuilder proxyBody = new StringBuilder();

            proxyBody.Append(_templateHeaderComment);

            proxyBody.Append(String.Format(_templateClassBegin, namespaceName, proxyClassName, proxyClassName,
                serverIpOrHostName, port, _bufferSize, _socketClientTimeout, _chunkSize, service.FullName));

            foreach (MethodInfo method in service.GetMethods())
            {
                if (method.DeclaringType.FullName == service.FullName
                    && method.ReturnType.FullName != "System.Void"
                    && method.GetCustomAttributesData().FirstOrDefault(a => a.ToString() == "[Kaushik.Spot.Library.SpotMethodAttribute()]") != null)
                {
                    string functionParameters = String.Empty;
                    string functionParameterNames = String.Empty;

                    ParameterInfo[] parameters = method.GetParameters();

                    if (parameters != null)
                    {
                        foreach (ParameterInfo parameter in parameters)
                        {
                            functionParameters += String.Format(" {0} {1},", parameter.ParameterType.FullName, parameter.Name);
                            functionParameterNames += String.Format(" {0},", parameter.Name);
                        }
                    }

                    if (functionParameterNames.Length > 0)
                    {
                        functionParameters = functionParameters.Substring(0, functionParameters.Length - 1);
                        functionParameterNames = functionParameterNames.Substring(0, functionParameterNames.Length - 1);
                    }

                    proxyBody.Append(String.Format(_templateMethodBody, method.ReturnType.ToString(), method.Name,
                        functionParameters, method.ReturnType.FullName, getDefaultValue(method.ReturnType), method.Name,
                        functionParameterNames, getConvertString(method.ReturnType)));
                }
            }

            proxyBody.Append(String.Format(_templateClassEnd));

            Debug.Print(proxyBody.ToString());

            return proxyBody.ToString();
        }

        private static string getConvertString(Type type)
        {
            return String.Format("Convert.To{0}", type.ToString().Replace("System.", ""));
        }

        private static string getDefaultValue(Type type)
        {
            return type.IsValueType ? (Activator.CreateInstance(type)).ToString() : "null";
        }

    }
}
