using System.Threading;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware.NetduinoPlus;
using System;
using Microsoft.SPOT;

namespace Kaushik.Spot.MicroServer
{
    /// <summary>
    /// This is a service that controls Switch points on Micro Controller
    /// </summary>
    public class SwitchService : ISwitchService
    {
        private static object _switch1;
        private static object _switch2;
        private static object _switch3;
        private static Thread _switchThread;

        /// <summary>
        /// Constructor
        /// </summary>
        static SwitchService()
        {
            _switchThread = new Thread(controlSwitch);
            _switch1 = false;
            _switch2 = false;
            _switch3 = false;

            if (!_switchThread.IsAlive)
            {
                _switchThread.Start();
            }
        }

        /// <summary>
        /// This function is used to Switch on and off given switch number.
        /// </summary>
        /// <param name="on">True for On, False for Off</param>
        /// <param name="switchNumber">Switch Number</param>
        /// <returns>True when done</returns>
        public bool ApplySwitchState(bool on, int switchNumber)
        {
            try
            {
                switch (switchNumber)
                {
                    case 1:
                        lock (_switch1)
                        {
                            _switch1 = on;
                        }
                        break;
                    case 2:
                        lock (_switch2)
                        {
                            _switch2 = on;
                        }
                        break;
                    case 3:
                        lock (_switch3)
                        {
                            _switch3 = on;
                        }
                        break;
                    default:
                        throw new NotSupportedException("Switch #" + switchNumber.ToString() + " is not supported.");
                }
            }
            catch (Exception error)
            {
                Debug.Print(error.Message);
                throw;
            }
            return true;
        }

        /// <summary>
        /// This function runs in a loop on a thread and is used to control switchs
        /// </summary>
        private static void controlSwitch()
        {
            OutputPort switchPin1 = new OutputPort(Pins.GPIO_PIN_D0, false);
            OutputPort switchPin2 = new OutputPort(Pins.GPIO_PIN_D1, false);
            OutputPort switchPin3 = new OutputPort(Pins.GPIO_PIN_D2, false);
            try
            {
                while (true)
                {
                    lock (_switch1)
                    {
                        switchPin1.Write((bool)_switch1);
                    }
                    lock (_switch2)
                    {
                        switchPin2.Write((bool)_switch2);
                    }
                    lock (_switch3)
                    {
                        switchPin3.Write((bool)_switch3);
                    }

                    Thread.Sleep(1000);      // wait 1000 ms
                }

            }
            catch (Exception error)
            {
                Debug.Print(error.Message);
                throw;
            }
        }


    }
}
